﻿
Imports System.Speech.Synthesis

Module Module_SpeechSynthesis

    Private TTS As SpeechSynthesizer

    Friend Sub TTS_Initialize()
        If TTS IsNot Nothing Then Return
        TTS = New SpeechSynthesizer
        TTS_SelectVoice("ENG")
        TTS_SetSpeed(0)
        TTS_SetVolume(80)
    End Sub

    Friend Sub TTS_CloseAll()
        If TTS Is Nothing Then Return
        TTS_Stop()
        TTS.Dispose()
    End Sub

    ' ======================================================================================
    '   PROCEDURES
    ' ======================================================================================
    Friend Sub TTS_SelectVoice(ByVal VoiceString As String)
        TTS_Stop()
        If TTS.GetInstalledVoices().Count = 0 Then Return
        Dim ThreeLetterISOLanguage As String
        Dim Gender As String = ""
        VoiceString = VoiceString.Trim.ToLower + "      "
        Gender = "male"
        ThreeLetterISOLanguage = VoiceString.Substring(0, 3)
        Select Case ThreeLetterISOLanguage
            Case "chi" : ThreeLetterISOLanguage = "zho"
        End Select
        If VoiceString.Substring(3).Contains("f") Then Gender = "female"
        If VoiceString.Substring(3).Contains("n") Then Gender = "neutral"
        Dim VoiceName As String = ""
        For i As Int32 = 0 To TTS.GetInstalledVoices().Count - 1
            With TTS.GetInstalledVoices()(i)
                If ThreeLetterISOLanguage = .VoiceInfo.Culture.ThreeLetterISOLanguageName.ToLower Then
                    VoiceName = .VoiceInfo.Name
                    If Gender = .VoiceInfo.Gender.ToString.ToLower Then
                        Exit For
                    End If
                End If
            End With
        Next
        If VoiceName <> "" Then
            TTS.SelectVoice(VoiceName)
        Else
            TTS.SelectVoice(TTS.GetInstalledVoices()(0).VoiceInfo.Name)
        End If
    End Sub

    'Friend Sub TTS_SelectVoice(ByVal VoiceIndex As Int32)
    '    TTS_Stop()
    '    If VoiceIndex < 1 Then
    '        VoiceIndex = 1
    '    End If
    '    If VoiceIndex > TTS.GetInstalledVoices().Count Then
    '        VoiceIndex = TTS.GetInstalledVoices().Count
    '    End If
    '    TTS.SelectVoice(TTS.GetInstalledVoices()(VoiceIndex - 1).VoiceInfo.Name)
    'End Sub

    Friend Sub TTS_SetSpeed(ByVal speed As Int32)
        TTS_Stop()
        If speed < -10 Then speed = -10
        If speed > 10 Then speed = 10
        TTS.Rate = speed
    End Sub

    Friend Sub TTS_SetVolume(ByVal volume As Int32)
        TTS_Stop()
        If volume < 0 Then volume = 0
        If volume > 100 Then volume = 100
        TTS.Volume = volume
    End Sub

    Friend Sub TTS_SpeakText(ByVal text As String)
        If TTS Is Nothing OrElse TTS.Volume = 0 Then Return
        TTS_Initialize()
        TTS.SpeakAsync(text)
    End Sub

    Friend Sub TTS_Stop()
        TTS_Initialize()
        TTS.SpeakAsyncCancelAll()
        Do
            Threading.Thread.Sleep(10)
        Loop Until TTS.State = SynthesizerState.Ready
    End Sub

    Friend Sub TTS_Pause()
        TTS_Initialize()
        TTS.Pause()
    End Sub

    Friend Sub TTS_Resume()
        TTS_Initialize()
        TTS.Resume()
    End Sub

    ' ======================================================================================
    '   FUNCTIONS
    ' ======================================================================================
    Friend Function TTS_GetInstalledVoices() As String
        TTS_Initialize()
        Dim index As Int32 = 1
        Dim s As String = vbCrLf
        s += "--------------------------------------------------------------" + vbCrLf
        s += " ISO   Culture  Gender   Age     Name                         " + vbCrLf
        s += "--------------------------------------------------------------" + vbCrLf
        For Each voice As InstalledVoice In TTS.GetInstalledVoices()
            s += " " + _
                voice.VoiceInfo.Culture.ThreeLetterISOLanguageName.ToUpper.PadRight(6) + _
                voice.VoiceInfo.Culture.ToString.ToUpper.PadRight(9) + _
                voice.VoiceInfo.Gender.ToString.PadRight(9) + _
                voice.VoiceInfo.Age.ToString().PadRight(8) + _
                voice.VoiceInfo.Name + vbCrLf
            index += 1
        Next
        s += "--------------------------------------------------------------" + vbCrLf
        Return s
    End Function

    Friend Function TTS_GetLanguage() As String
        TTS_Initialize()
        Return TTS.Voice.Culture.ThreeLetterISOLanguageName
    End Function

    Friend Function TTS_SynthesizerReady() As Boolean
        TTS_Initialize()
        Return TTS.State = SynthesizerState.Ready
    End Function

    Friend Function TTS_SynthesizerPaused() As Boolean
        TTS_Initialize()
        Return TTS.State = SynthesizerState.Ready
    End Function

    Friend Function TTS_SynthesizerSpeaking() As Boolean
        TTS_Initialize()
        Return TTS.State = SynthesizerState.Speaking
    End Function

End Module
